UNIT mPOP3Client;

{******************************************************************
  mPOP3Client.pas

  Klasse zum Abruf von E-Mails.
  Diese Datei gehrt zu den Materialien zu dem Workshop
  "Zentralabitur: Kommunikation in Netzwerken" des 5. Informatiktag
  in Paderborn. Sie darf fr eigene Unterrichtszwecke frei verwendet
  werden. Eine anderweitige Verffentlichung ist nur mit meiner Erlaubnis
  zulssig.

  Bitte beachten Sie die Hinweise in der Projektdatei
  pMailClient.dpr bzgl. der hier verwendeten Klasse Verbindung.

  (c) Joachim Deckers, <joachim@deckers.name>, 03.04.2006
******************************************************************}

interface

//--------------------  ggf Uses-Liste anpassen !  --------------------
uses mSuM,mTextwerkzeug,mVerbindung;

const CRLF = #13#10;

type
  POP3Client = class

  protected //Attribute
    zAnzahlNachrichten : GanzeZahl;
    hatVerbindung : Verbindung;
    hatTextwerkzeug: Textwerkzeug;
    zAngemeldet : Wahrheitswert;
    zFehlermeldung : Zeichenkette;

  public //Methoden
    constructor init; virtual;
    procedure abmelden; virtual;
    procedure anmelden (pServer: Zeichenkette; pPort: GanzeZahl; pUsername: Zeichenkette; pKennwort: Zeichenkette); virtual;
    function angemeldet : Wahrheitswert; virtual;
    function anzahlNachrichten : GanzeZahl; virtual;
    function fehler : Wahrheitswert; virtual;
    function fehlermeldung : Zeichenkette; virtual;
    function gibNachricht (pNummer: GanzeZahl) : Zeichenkette; virtual;
    destructor gibFrei; virtual;

   end;

implementation

//+---------------------------------------------------------------------
//|         POP3Client: Methodendefinition 
//+---------------------------------------------------------------------

//-------- init (public) -----------------------------------------------
constructor POP3Client.init;
begin
  hatVerbindung      := nil;
  zAngemeldet        := FALSCH;
  zAnzahlNachrichten := 0;
  zFehlermeldung     := '';
  hatTextwerkzeug    := Textwerkzeug.init;
end;

//-------- abmelden (public) -------------------------------------------
procedure POP3Client.abmelden;
begin
  if zAngemeldet then 
    hatVerbindung.sende('QUIT'+CRLF);
  if hatVerbindung<>nil then begin
    hatVerbindung.gibFrei;
    hatVerbindung:=nil;
    zAngemeldet:=FALSCH;
  end;
end;

//-------- anmelden (public) -------------------------------------------
procedure POP3Client.anmelden (pServer: Zeichenkette; pPort: GanzeZahl; pUsername: Zeichenkette; pKennwort: Zeichenkette);
var lNachricht: Zeichenkette;    
begin
  zFehlermeldung:='';
  zAngemeldet:=FALSCH;
  zAnzahlNachrichten:=0;
  hatVerbindung := Verbindung.init(pServer,pPort,FALSCH);
  if hatVerbindung.vorhanden then 
  begin
    lNachricht := hatVerbindung.empfangeneNachricht;
    if hatTextWerkzeug.positionVon(lNachricht,'+OK')=0 then
      zFehlermeldung := lNachricht
    else
    begin
      hatVerbindung.sende('USER '+pUsername+CRLF);
      lNachricht := hatVerbindung.empfangeneNachricht;
      if hatTextWerkzeug.positionVon(lNachricht,'+OK')=0 then
        zFehlermeldung := lNachricht
      else
      begin
        hatVerbindung.sende('PASS '+pKennwort+CRLF);
        lNachricht := hatVerbindung.empfangeneNachricht;
        if hatTextWerkzeug.positionVon(lNachricht,'+OK')=0 then
          zFehlermeldung := lNachricht
        else
        begin
          hatVerbindung.sende('STAT'+CRLF);
          lNachricht := hatVerbindung.empfangeneNachricht;
          if hatTextWerkzeug.positionVon(lNachricht,'+OK')=0 then
            zFehlermeldung := lNachricht
          else begin
            zAnzahlNachrichten := hatTextwerkzeug.alsGanzeZahl(hatTextwerkzeug.wortAn(lNachricht,2))    ;
            zAngemeldet:=WAHR; 
          end 
        end  
      end
    end
  end 
  else 
    zFehlermeldung := 'Keine Verbindung!';
end;

//-------- angemeldet (public) -----------------------------------------
function POP3Client.angemeldet : Wahrheitswert;
begin
  result := zAngemeldet
end;

//-------- anzahlNachrichten (public) ----------------------------------
function POP3Client.anzahlNachrichten : GanzeZahl;
begin
  result := zAnzahlNachrichten
end;

//-------- fehler (public) ---------------------------------------------
function POP3Client.fehler : Wahrheitswert;
begin
  result := zFehlermeldung <> ''
end;

//-------- fehlermeldung (public) --------------------------------------
function POP3Client.fehlermeldung : Zeichenkette;
begin
  result := zFehlermeldung;
end;

//-------- gibNachricht (public) ---------------------------------------
function POP3Client.gibNachricht (pNummer: GanzeZahl) : Zeichenkette;
var lNachricht, lMail: Zeichenkette;
begin
  zFehlermeldung:=''; lMail:='';
  if zAngemeldet then begin
    if (pNummer>0) and (pNummer<=zAnzahlNachrichten) then
    begin
      hatVerbindung.sende('RETR '+hatTextwerkzeug.alsText(pNummer)+CRLF);
      lNachricht := hatVerbindung.empfangeneNachricht;
      if hatTextwerkzeug.positionVon(lNachricht,'+OK')<>0 then
      begin
        lNachricht := hatVerbindung.empfangeneNachricht;
        while lNachricht<>'.'+CRLF do begin
          if hatTextwerkzeug.zeichenAn(lNachricht,1)='.' then
            lNachricht:=hatTextwerkzeug.teilZeichenkette(lNachricht,2,hatTextwerkzeug.laenge(lNachricht));
          lMail := lMail + lNachricht;
          lNachricht := hatVerbindung.empfangeneNachricht;
        end;
      end
      else
        zFehlermeldung := lNachricht;
    end
    else
      zFehlermeldung := 'Unzulssiger Nachrichtenindex!';
  end;
  result := lMail;   
end;

//-------- gibFrei (public) --------------------------------------------
destructor POP3Client.gibFrei;
begin
  //hier ggf. Code ergnzen
end;

end.
