{*******************************************************************************
 *  Verbindung : Netzwerkuntersttzung fr Stifte und Muse                    *
 *               Version 2.0 vom 27.02.2006                                    *
 *               Joachim Deckers, Mariengymnasium Arnsberg                     *
 *               Ursprngliche Fassung: Klaus Dingemann                        *
 *               eMail: sum-delphi@deckers.name                                *
 *******************************************************************************
 * Bitte teilen Sie den Entwicklern Ihre Verbesserungsvorschlge mit oder      *
 * weisen Sie sie auf Fehler (und ggf. deren Berichtigung) hin.                *
 *                                                                             *
 * Danke fr Ihre Mitarbeit.                                                   *
 *******************************************************************************
 * nderungen                                                                  *
 * Version 2.0:                                                                *
 *   - Die Zeilenendezeichen am Ende einer empfangenen Nachricht werden nun    *
 *     mit bergeben. Dadurch knnen leere Nachrichten von Leerzeilen unter-   *
 *     schieden werden, auerdem knnen noch nicht abgeschlossene Zeilen sehr  *
 *     langer Nachrichten als solche erkannt werden.                           *
 *   - Einfhrung eines "TimeOut" fr empfangeneNachricht: Falls 20 Sekunden   *
 *     verstrichen sind, ohne dass eine Nachricht empfangen wurde, liefert     *
 *     empfangene Nachricht eine leere Zeichenkette zurck. Potentielle Endlos-*
 *     schleifen werden dadurch verhindert.                                    *
 *   - Beim Senden einer Nachricht wird kein Zeilenendezeichen mehr automatisch*
 *     angehngt.                                                              *
 *******************************************************************************}

unit mVerbindung;

interface

uses  ScktComp, Forms, Windows, Messages, SysUtils, Classes, Graphics, Controls,
      Dialogs, StDCtrls, mProtokollfenster, mListe, mTextwerkzeug, mSum;
const
    ZEILENENDE=#13#10;
    TIMEOUT : Zahl = 20 ; // 20 Sekunden Timeout fr "empfangeneNachricht"

type
  ZeichenketteObjekt = class
                  private
                   zInhalt:Zeichenkette;
                  public
                   constructor init(pInhalt:Zeichenkette);
                   procedure setzeInhalt(pInhalt:Zeichenkette);
                   function Inhalt:Zeichenkette;
                   destructor gibFrei;
                 end;

  Verbindung = class
                private
                 hatSocket:TClientSocket;
                 zMitProtokoll, zAngemeldet:Wahrheitswert;
                 hatClientKontrollfenster:TKontrollfenster;
                 zServerPort, zClientport:GanzeZahl;
                 zServerAdresse, zClientAdresse:Zeichenkette;
                 zNachricht:Zeichenkette;
                 zFehler:Wahrheitswert;
                 hatTextwerkzeug:Textwerkzeug;
                protected
                 hatListe:Liste;
                 procedure ClientSocketRead(Sender: TObject;Socket: TCustomWinSocket); virtual;
                 procedure ClientSocketError(Sender: TObject;Socket: TCustomWinSocket; ErrorEvent: TErrorEvent; var ErrorCode: GanzeZahl);
                 procedure ClientSocketConnect(Sender: TObject;Socket: TCustomWinSocket);
                public
                 constructor init(pIPAdresse:Zeichenkette; pPortNr:GanzeZahl; pMitProtokollieren:Wahrheitswert);
                 procedure sende(pNachricht:Zeichenkette);virtual;
                 function empfangeneNachricht:Zeichenkette;
                 function partnerAdresse:Zeichenkette;
                 function eigeneAdresse:Zeichenkette;
                 function partnerPort:GanzeZahl;
                 function eigenerPort:GanzeZahl;
                 function vorhanden:Wahrheitswert;
                 function mitProtokoll:Wahrheitswert;
                 destructor gibFrei; virtual;
                end;
implementation

    constructor ZeichenketteObjekt.init(pInhalt:Zeichenkette);
    begin
      zInhalt:=pInhalt;
    end;

    procedure ZeichenketteObjekt.setzeInhalt(pInhalt:Zeichenkette);
    begin
      zInhalt:=pInhalt;
    end;

    function ZeichenketteObjekt.Inhalt:Zeichenkette;
    begin
      result:=zInhalt;
    end;

    destructor ZeichenketteObjekt.gibFrei;
    begin
    end;

   constructor Verbindung.init(pIPAdresse:Zeichenkette; pPortNr:GanzeZahl; pMitProtokollieren:Wahrheitswert);
   begin
    hatListe:=Liste.init;
    hatTextwerkzeug:=Textwerkzeug.init;
    hatSocket:=TClientSocket.Create(nil);
    hatSocket.Host:=pIPAdresse;
    hatSocket.Port:=pPortNr;
    zMitProtokoll:=pMitProtokollieren;
    zFehler:=false;
    hatSocket.onError:=clientSocketError;
    hatSocket.onRead:=ClientSocketRead;
    hatSocket.onConnect:=ClientSocketConnect;
    hatSocket.open;
    if zMitProtokoll then begin
       hatClientKontrollfenster:=TKontrollfenster.create(Application);
       hatClientKontrollfenster.show;
       hatClientKontrollfenster.Top:=300;
    end;
    while not hatSocket.socket.connected and not zFehler do
      application.processMessages;
  end;

  procedure Verbindung.sende(pNachricht:Zeichenkette);
  begin
    hatSocket.socket.sendText(pNachricht);
    zNachricht:='';
    if zMitProtokoll then
     hatClientKontrollfenster.memo1.lines.add('Verbindung sendet '+pNachricht);
  end;

  function Verbindung.empfangeneNachricht:Zeichenkette;
  var
    lNachricht:Zeichenkette;
    lStartzeit:Zahl;
  begin
    lStartzeit := GetTime();
    while (hatListe.istLeer) and ((GetTime() - lStartzeit) < TIMEOUT* 1.0/(24*60*60) ) do
      application.processMessages;
    if not hatListe.istLeer then begin
      hatListe.zumAnfang;
      lNachricht:=ZeichenketteObjekt(hatListe.aktuelles).Inhalt;
      result:=lNachricht;
      hatListe.entferneAktuelles;
    end else
      result := '';  
  end;

  function Verbindung.mitProtokoll:Wahrheitswert;
  begin
    result:=zMitProtokoll;
  end;

  function Verbindung.partnerAdresse:Zeichenkette;
  begin
    result:=zServerAdresse;
  end;

  function Verbindung.partnerPort:GanzeZahl;
  begin
   result:=zServerPort;
  end;

  function Verbindung.eigeneAdresse:Zeichenkette;
  begin
    result:=zClientAdresse;
  end;

  function Verbindung.eigenerPort:GanzeZahl;
  begin
    result:=zClientport
  end;

  function Verbindung.vorhanden:Wahrheitswert;
  begin
    result:=self.hatSocket.socket.connected;
  end;

  destructor Verbindung.gibFrei;
  begin
    hatSocket.close;
    zAngemeldet:=false;
    hatClientKontrollfenster.free;
  end;

  procedure Verbindung.ClientSocketRead(Sender: TObject;Socket: TCustomWinSocket);
  var
    lNachricht, lEineNachricht:Zeichenkette;
    lPos:GanzeZahl;
  begin
    lNachricht:=Socket.receiveText;

    if zMitProtokoll then
        hatClientKontrollfenster.memo1.lines.add('Verbindung liest '+lNachricht);

    repeat
      lPos := hatTextwerkzeug.positionVon(lNachricht,ZEILENENDE);
      if lPos>0 then begin
        lEineNachricht:=hatTextwerkzeug.teilzeichenkette(lNachricht,1,lPos+hatTextwerkzeug.laenge(ZEILENENDE)-1);
        lNachricht := hatTextwerkzeug.textOhne(lNachricht,1,lPos+hatTextwerkzeug.laenge(ZEILENENDE)-1);
      end else // Zeile ohne Zeilenendezeichen gefunden
      begin
        lEineNachricht := lNachricht;
        lNachricht := ''
      end;
      hatListe.haengeAn(ZeichenketteObjekt.init(lEineNachricht));
    until lNachricht='';

  end;


  procedure Verbindung.ClientSocketConnect(Sender: TObject;Socket: TCustomWinSocket);
  begin
    zServerAdresse:=hatSocket.socket.remoteAddress;
    zServerPort:=hatSocket.socket.remotePort;
    zClientAdresse:=hatSocket.socket.localAddress;
    zClientPort:=hatSocket.socket.localPort;
    zAngemeldet:=true;
    if zMitProtokoll then
       hatClientKontrollfenster.memo1.lines.add(zClientAdresse +' ist an Port '+intToStr(zClientPort)+' mit Server '+zServerAdresse+' an Port '+intToStr(zServerPort)+' verbunden.');
  end;

  procedure Verbindung.ClientSocketError(Sender: TObject;Socket: TCustomWinSocket; ErrorEvent: TErrorEvent; var ErrorCode: GanzeZahl);
  var
    lFehler:Zeichenkette;
  begin
    case ErrorEvent of
    eeGeneral:lFehler:='Unbekannter Fehler';
    eeSend:lFehler:='Der Client konnte nicht senden';
    eeReceive:lFehler:='Der Client konnte keine Nachricht empfangen';
    eeConnect:lFehler:='Der Client konnte keine Verbindung aufbauen';
    eeDisconnect:lFehler:='Die Verbindung konnte nicht geschlossen werden.';
    eeAccept:lFehler:='Der Client konnte keine Verbindung aufbauen';
   else
      lFehler:='Verbindungsfehler';
    end;
    if zMitProtokoll then
       hatClientKontrollfenster.memo1.lines.add(lFehler);
    zFehler:=true;
  end;

end.
